searchNodes=[{"type":"module","title":"ExcellentMigrations.CredoCheck.MigrationsSafety","doc":"This check has a base priority of high and works with any version of Elixir. Explanation Checks if database migrations contain potentially dangerous operations. Check-Specific Parameters There are no specific parameters for this check. General Parameters Like with all checks, general params can be applied. Parameters can be configured via the .credo.exs config file .","ref":"ExcellentMigrations.CredoCheck.MigrationsSafety.html"},{"type":"module","title":"ExcellentMigrations.DangersDetector","doc":"This module finds potentially dangerous or destructive database operations in a given migration AST.","ref":"ExcellentMigrations.DangersDetector.html"},{"type":"function","title":"ExcellentMigrations.DangersDetector.detect_dangers/2","doc":"Traverses ast and finds potentially dangerous database operations. Returns keyword list containing danger types and lines where they were detected. Parameters ast is a structure that represents AST of database migration. It can be obtained e.g. via Code.string_to_quoted!/1 . Examples iex&gt; source_code = &quot;&quot;&quot; ...&gt; alter table(&quot;dumplings&quot;) do ...&gt; remove(:taste, :string) ...&gt; remove(:stuffing, :string) ...&gt; end ...&gt; &quot;&quot;&quot; iex&gt; ast = Code . string_to_quoted! ( source_code ) { :ok , { :alter , [ line : 1 ] , [ { :table , [ line : 1 ] , [ &quot;dumplings&quot; ] } , [ do : { :__block__ , [ ] , [ { :remove , [ line : 2 ] , [ :taste , :string ] } , { :remove , [ line : 3 ] , [ :stuffing , :string ] } ] } ] ] } } iex&gt; ExcellentMigrations.DangersDetector . detect_dangers ( ast , source_code ) [ column_removed : 2 , column_removed : 3 ]","ref":"ExcellentMigrations.DangersDetector.html#detect_dangers/2"},{"type":"type","title":"ExcellentMigrations.DangersDetector.ast/0","doc":"","ref":"ExcellentMigrations.DangersDetector.html#t:ast/0"},{"type":"type","title":"ExcellentMigrations.DangersDetector.danger_type/0","doc":"","ref":"ExcellentMigrations.DangersDetector.html#t:danger_type/0"},{"type":"type","title":"ExcellentMigrations.DangersDetector.line/0","doc":"","ref":"ExcellentMigrations.DangersDetector.html#t:line/0"},{"type":"module","title":"ExcellentMigrations.Runner","doc":"This module finds migration files in a project and detects potentially dangerous database operations in them.","ref":"ExcellentMigrations.Runner.html"},{"type":"function","title":"ExcellentMigrations.Runner.check_migrations/1","doc":"Detects potentially dangerous database operations in database migration files. Options :migrations_paths - optional list of file paths to be checked. Scope of analysis If migrations_paths are specified, the analysis will be narrowed down to these files only. If not and application env :excellent_migrations, :start_after is set, only migrations with timestamp older than the provided one will be chosen. If none of the above, all migration files in a project will be analyzed.","ref":"ExcellentMigrations.Runner.html#check_migrations/1"},{"type":"type","title":"ExcellentMigrations.Runner.danger/0","doc":"","ref":"ExcellentMigrations.Runner.html#t:danger/0"},{"type":"type","title":"ExcellentMigrations.Runner.danger_type/0","doc":"","ref":"ExcellentMigrations.Runner.html#t:danger_type/0"},{"type":"task","title":"Mix.Tasks.ExcellentMigrations.CheckSafety","doc":"Runs analysis on database migration files and detects potentially dangerous operations.","ref":"Mix.Tasks.ExcellentMigrations.CheckSafety.html"},{"type":"function","title":"Mix.Tasks.ExcellentMigrations.CheckSafety.run/1","doc":"Callback implementation for Mix.Task.run/1 .","ref":"Mix.Tasks.ExcellentMigrations.CheckSafety.html#run/1"},{"type":"task","title":"Mix.Tasks.ExcellentMigrations.Migrate","doc":"Runs ecto.migrate task only if no dangerous operations were detected in migrations.","ref":"Mix.Tasks.ExcellentMigrations.Migrate.html"},{"type":"function","title":"Mix.Tasks.ExcellentMigrations.Migrate.run/1","doc":"Callback implementation for Mix.Task.run/1 .","ref":"Mix.Tasks.ExcellentMigrations.Migrate.html#run/1"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"Changelog - Unreleased","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"Changelog - 0.1.8 - 2024-02-09","doc":"Fixed Using Logger.warning instead of deprecated Logger.warn","ref":"changelog.html#0-1-8-2024-02-09"},{"type":"extras","title":"Changelog - 0.1.7 - 2024-01-18","doc":"Fixed Correct exit code Choosing the right type when detected lack of disable_ddl_transaction or disable_migration_lock","ref":"changelog.html#0-1-7-2024-01-18"},{"type":"extras","title":"Changelog - 0.1.6 - 2022-11-20","doc":"Added Detecting volatile default when adding a column or adding a default Fixed Detecting index with too many columns","ref":"changelog.html#0-1-6-2022-11-20"},{"type":"extras","title":"Changelog - 0.1.5 - 2022-08-05","doc":"Added Handling unique index Detecting concurrently added indexes without disable_ddl_transaction nor disable_migration_lock","ref":"changelog.html#0-1-5-2022-08-05"},{"type":"extras","title":"Changelog - 0.1.4 - 2022-07-24","doc":"Fixed Handling &quot;if (not) exists&quot; variants of operations","ref":"changelog.html#0-1-4-2022-07-24"},{"type":"extras","title":"Changelog - 0.1.3 - 2021-12-07","doc":"Added Safety assured handled with config comments Deprecated Using module attribute @safety_assured","ref":"changelog.html#0-1-3-2021-12-07"},{"type":"extras","title":"Changelog - 0.1.2 - 2021-11-25","doc":"Fixed Fixed generating warning messages","ref":"changelog.html#0-1-2-2021-11-25"},{"type":"extras","title":"Changelog - 0.1.1 - 2021-11-19","doc":"Fixed Fixed handling index with one column as an atom","ref":"changelog.html#0-1-1-2021-11-19"},{"type":"extras","title":"Changelog - 0.1.0 - 2021-11-18","doc":"Added First release!","ref":"changelog.html#0-1-0-2021-11-18"},{"type":"extras","title":"License","doc":"Copyright (c) 2021 Artur Sulej Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"Detect potentially dangerous or destructive operations in your database migrations.","ref":"readme.html"},{"type":"extras","title":"Overview - Installation","doc":"The package can be installed by adding :excellent_migrations to your list of dependencies in mix.exs : def deps do [ { :excellent_migrations , &quot;~&gt; 0.1&quot; , only : [ :dev , :test ] , runtime : false } ] end","ref":"readme.html#installation"},{"type":"extras","title":"Overview - Documentation","doc":"Documentation is available on Hexdocs .","ref":"readme.html#documentation"},{"type":"extras","title":"Overview - How It Works","doc":"This tool analyzes code (AST) of migration files. You don't have to edit or include any additional code in your migration files, except for occasionally adding a config comment for assuring safety .","ref":"readme.html#how-it-works"},{"type":"extras","title":"Overview - How to use it","doc":"There are multiple ways to integrate with Excellent Migrations. Credo check Excellent Migrations provide custom, ready-to-use check for Credo . Add ExcellentMigrations.CredoCheck.MigrationsSafety to your .credo file: %{ configs : [ %{ # … checks : [ # … { ExcellentMigrations.CredoCheck.MigrationsSafety , [ ] } ] } ] } Example credo warnings: Warnings - please take a look ┃ ┃ [ W ] ↗ Raw SQL used ┃ apps / cookbook / priv / repo / migrations / 20211024133700 _create_recipes . exs : 13 #(Cookbook.Repo.Migrations.CreateRecipes.up) ┃ [ W ] ↗ Index added not concurrently ┃ apps / cookbook / priv / repo / migrations / 20211024133705 _create_index_on_veggies . exs : 37 #(Cookbook.Repo.Migrations.CreateIndexOnVeggies.up) mix task mix excellent_migrations.check_safety This mix task analyzes migrations and logs a warning for each danger detected. migration task mix excellent_migrations.migrate Running this task will first analyze migrations. If no dangers are detected it will proceed and run mix ecto.migrate . If there are any, it will log errors and stop. Code You can also use it in code. To do so, you need to get source code and AST of your migration file, e.g. via File.read!/1 and Code.string_to_quoted/2 . Then pass them to ExcellentMigrations.DangersDetector.detect_dangers(ast) . It will return a keyword list containing danger types and lines where they were detected.","ref":"readme.html#how-to-use-it"},{"type":"extras","title":"Overview - Checks","doc":"Potentially dangerous operations: Adding a check constraint Adding a column with a default value Backfilling data Column with volatile default Changing the type of a column Executing SQL directly Removing a column Renaming a column Renaming a table Setting NOT NULL on an existing column Postgres-specific checks: Adding a json column Adding a reference Adding an index non-concurrently Adding an index concurrently without disabling lock or transaction Best practices: Keeping non-unique indexes to three columns or less You can also disable specific checks . Removing a column If Ecto is still configured to read a column in any running instances of the application, then queries will fail when loading data into your structs. This can happen in multi-node deployments or if you start the application before running migrations. BAD ❌ # Without a code change to the Ecto Schema def change do alter table ( &quot;recipes&quot; ) do remove :no_longer_needed_column end end GOOD ✅ Safety can be assured if the application code is first updated to remove references to the column so it's no longer loaded or queried. Then, the column can safely be removed from the table. Deploy code change to remove references to the field. Deploy migration change to remove the column. First deployment: # First deploy, in the Ecto schema defmodule Cookbook.Recipe do schema &quot;recipes&quot; do - column :no_longer_needed_column, :text end end Second deployment: def change do alter table ( &quot;recipes&quot; ) do remove :no_longer_needed_column end end Adding a column with a default value Adding a column with a default value to an existing table may cause the table to be rewritten. During this time, reads and writes are blocked in Postgres, and writes are blocked in MySQL and MariaDB. BAD ❌ Note: This becomes safe in: Postgres 11+ MySQL 8.0.12+ MariaDB 10.3.2+ def change do alter table ( &quot;recipes&quot; ) do add :favourite , :boolean , default : false # This took 10 minutes for 100 million rows with no fkeys, # Obtained an AccessExclusiveLock on the table, which blocks reads and # writes. end end GOOD ✅ Add the column first, then alter it to include the default. First migration: def change do alter table ( &quot;recipes&quot; ) do add :favourite , :boolean # This took 0.27 milliseconds for 100 million rows with no fkeys, end end Second migration: def change do alter table ( &quot;recipes&quot; ) do modify :favourite , :boolean , default : false # This took 0.28 milliseconds for 100 million rows with no fkeys, end end Schema change to read the new column: schema &quot;recipes&quot; do + field :favourite, :boolean, default: false end Column with volatile default If the default value is volatile (e.g., clock_timestamp() , uuid_generate_v4() , random() ) each row will need to be updated with the value calculated at the time ALTER TABLE is executed. BAD ❌ Adding volatile default to column: def change do alter table ( :recipes ) do modify ( :identifier , :uuid , default : fragment ( &quot;uuid_generate_v4()&quot; ) ) end end Adding column with volatile default: def change do alter table ( :recipes ) do add ( :identifier , :uuid , default : fragment ( &quot;uuid_generate_v4()&quot; ) ) end end GOOD ✅ To avoid a potentially lengthy update operation, particularly if you intend to fill the column with mostly nondefault values anyway, it may be preferable to: add the column with no default insert the correct values using UPDATE query only then add any desired default Also creating a new table with column with volatile default is safe, because it does not contain any records. Backfilling data Ecto creates a transaction around each migration, and backfilling in the same transaction that alters a table keeps the table locked for the duration of the backfill. Also, running a single query to update data can cause issues for large tables. BAD ❌ defmodule Cookbook.BackfillRecipes do use Ecto.Migration import Ecto.Query def change do alter table ( &quot;recipes&quot; ) do add :new_data , :text end flush ( ) Cookbook.Recipe |&gt; where ( new_data : nil ) |&gt; Cookbook.Repo . update_all ( set : [ new_data : &quot;some data&quot; ] ) end end GOOD ✅ There are several different strategies to perform safe backfilling. This article explains them in great details. Changing the type of a column Changing the type of a column may cause the table to be rewritten. During this time, reads and writes are blocked in Postgres, and writes are blocked in MySQL and MariaDB. BAD ❌ Safe in Postgres: increasing length on varchar or removing the limit changing varchar to text changing text to varchar with no length limit Postgres 9.2+ - increasing precision (NOTE: not scale) of decimal or numeric columns. eg, increasing 8,2 to 10,2 is safe. Increasing 8,2 to 8,4 is not safe. Postgres 9.2+ - changing decimal or numeric to be unconstrained Postgres 12+ - changing timestamp to timestamptz when session TZ is UTC Safe in MySQL/MariaDB: increasing length of varchar from &lt; 255 up to 255. increasing length of varchar from &gt; 255 up to max. def change do alter table ( &quot;recipes&quot; ) do modify :my_column , :boolean , from : :text end end GOOD ✅ Take a phased approach: Create a new column In application code, write to both columns Backfill data from old column to new column In application code, move reads from old column to the new column In application code, remove old column from Ecto schemas. Drop the old column. Renaming a column Ask yourself: &quot;Do I really need to rename a column?&quot;. Probably not, but if you must, read on and be aware it requires time and effort. If Ecto is configured to read a column in any running instances of the application, then queries will fail when loading data into your structs. This can happen in multi-node deployments or if you start the application before running migrations. There is a shortcut: Don't rename the database column, and instead rename the schema's field name and configure it to point to the database column. BAD ❌ # In your schema schema &quot;recipes&quot; do field :summary , :text end # In your migration def change do rename table ( &quot;recipes&quot; ) , :title , to : :summary end The time between your migration running and your application getting the new code may encounter trouble. GOOD ✅ Strategy 1 Rename the field in the schema only, and configure it to point to the database column and keep the database column the same. Ensure all calling code relying on the old field name is also updated to reference the new field name. defmodule Cookbook.Recipe do use Ecto.Schema schema &quot;recipes&quot; do field :author , :string field :preparation_minutes , :integer , source : :prep_min end end ## Update references in other parts of the codebase: recipe = Repo.get(Recipe, &quot;my_id&quot;) - recipe.prep_min + recipe.preparation_minutes Strategy 2 Take a phased approach: Create a new column In application code, write to both columns Backfill data from old column to new column In application code, move reads from old column to the new column In application code, remove old column from Ecto schemas. Drop the old column. Renaming a table Ask yourself: &quot;Do I really need to rename a table?&quot;. Probably not, but if you must, read on and be aware it requires time and effort. If Ecto is still configured to read a table in any running instances of the application, then queries will fail when loading data into your structs. This can happen in multi-node deployments or if you start the application before running migrations. There is a shortcut: rename the schema only, and do not change the underlying database table name. BAD ❌ def change do rename table ( &quot;recipes&quot; ) , to : table ( &quot;dish_algorithms&quot; ) end GOOD ✅ Strategy 1 Rename the schema only and all calling code, and don’t rename the table: - defmodule Cookbook.Recipe do + defmodule Cookbook.DishAlgorithm do use Ecto.Schema schema &quot;dish_algorithms&quot; do field :author, :string field :preparation_minutes, :integer end end # and in calling code: - recipe = Cookbook.Repo.get(Cookbook.Recipe, &quot;my_id&quot;) + dish_algorithm = Cookbook.Repo.get(Cookbook.DishAlgorithm, &quot;my_id&quot;) Strategy 2 Take a phased approach: Create the new table. This should include creating new constraints (checks and foreign keys) that mimic behavior of the old table. In application code, write to both tables, continuing to read from the old table. Backfill data from old table to new table In application code, move reads from old table to the new table In application code, remove the old table from Ecto schemas. Drop the old table. Adding a check constraint Adding a check constraint blocks reads and writes to the table in Postgres, and blocks writes in MySQL/MariaDB while every row is checked. BAD ❌ def change do create constraint ( &quot;ingredients&quot; , :price_must_be_positive , check : &quot;price &gt; 0&quot; ) # Creating the constraint with validate: true (the default when unspecified) # will perform a full table scan and acquires a lock preventing updates end GOOD ✅ There are two operations occurring: Creating a new constraint for new or updating records Validating the new constraint for existing records If these commands are happening at the same time, it obtains a lock on the table as it validates the entire table and fully scans the table. To avoid this full table scan, we can separate the operations. In one migration: def change do create constraint ( &quot;ingredients&quot; , :price_must_be_positive , check : &quot;price &gt; 0&quot; , validate : false ) # Setting validate: false will prevent a full table scan, and therefore # commits immediately. end In the next migration: def change do execute &quot;ALTER TABLE ingredients VALIDATE CONSTRAINT price_must_be_positive&quot; , &quot;&quot; # Acquires SHARE UPDATE EXCLUSIVE lock, which allows updates to continue end These can be in the same deployment, but ensure there are 2 separate migrations. Setting NOT NULL on an existing column Setting NOT NULL on an existing column blocks reads and writes while every row is checked. Just like the Adding a check constraint scenario, there are two operations occurring: Creating a new constraint for new or updating records Validating the new constraint for existing records To avoid the full table scan, we can separate these two operations. BAD ❌ def change do alter table ( &quot;recipes&quot; ) do modify :favourite , :boolean , null : false end end GOOD ✅ Add a check constraint without validating it, backfill data to satiate the constraint and then validate it. This will be functionally equivalent. In the first migration: # Deployment 1 def change do create constraint ( &quot;recipes&quot; , :favourite_not_null , check : &quot;favourite IS NOT NULL&quot; , validate : false ) end This will enforce the constraint in all new rows, but not care about existing rows until that row is updated. You'll likely need a data migration at this point to ensure that the constraint is satisfied. Then, in the next deployment's migration, we'll enforce the constraint on all rows: # Deployment 2 def change do execute &quot;ALTER TABLE recipes VALIDATE CONSTRAINT favourite_not_null&quot; , &quot;&quot; end If you're using Postgres 12+, you can add the NOT NULL to the column after validating the constraint. From the Postgres 12 docs: SET NOT NULL may only be applied to a column provided none of the records in the table contain a NULL value for the column. Ordinarily this is checked during the ALTER TABLE by scanning the entire table; however, if a valid CHECK constraint is found which proves no NULL can exist, then the table scan is skipped. # **Postgres 12+ only** def change do execute &quot;ALTER TABLE recipes VALIDATE CONSTRAINT favourite_not_null&quot; , &quot;&quot; alter table ( &quot;recipes&quot; ) do modify :favourite , :boolean , null : false end drop constraint ( &quot;recipes&quot; , :favourite_not_null ) end If your constraint fails, then you should consider backfilling data first to cover the gaps in your desired data integrity, then revisit validating the constraint. Executing SQL directly Excellent Migrations can’t ensure safety for raw SQL statements. Make really sure that what you’re doing is safe, then use: defmodule Cookbook.ExecuteRawSql do # excellent_migrations:safety-assured-for-this-file raw_sql_executed def change do execute ( &quot;...&quot; ) end end Adding an index non-concurrently Creating an index will block both reads and writes. BAD ❌ def change do create index ( &quot;recipes&quot; , [ :slug ] ) # This obtains a ShareLock on &quot;recipes&quot; which will block writes to the table end GOOD ✅ With Postgres, instead create the index concurrently which does not block reads. You will need to disable the database transactions to use CONCURRENTLY , and since Ecto obtains migration locks through database transactions this also implies that competing nodes may attempt to try to run the same migration (eg, in a multi-node Kubernetes environment that runs migrations before startup). Therefore, some nodes will fail startup for a variety of reasons. @disable_ddl_transaction true @disable_migration_lock true def change do create index ( &quot;recipes&quot; , [ :slug ] , concurrently : true ) end The migration may still take a while to run, but reads and updates to rows will continue to work. For example, for 100,000,000 rows it took 165 seconds to add run the migration, but SELECTS and UPDATES could occur while it was running. Do not have other changes in the same migration ; only create the index concurrently and separate other changes to later migrations. Adding an index concurrently without disabling lock or transaction Concurrently indexes need to set both @disable_ddl_transaction and @disable_migration_lock to true. See more : BAD ❌ defmodule Cookbook.AddIndex do def change do create index ( :recipes , [ :cookbook_id , :cuisine ] , concurrently : true ) end end GOOD ✅ defmodule Cookbook.AddIndex do @disable_ddl_transaction true @disable_migration_lock true def change do create index ( :recipes , [ :cookbook_id , :cuisine ] , concurrently : true ) end end Adding a reference Adding a foreign key blocks writes on both tables. BAD ❌ def change do alter table ( &quot;recipes&quot; ) do add :cookbook_id , references ( &quot;cookbooks&quot; ) end end GOOD ✅ In the first migration def change do alter table ( &quot;recipes&quot; ) do add :cookbook_id , references ( &quot;cookbooks&quot; , validate : false ) end end In the second migration def change do execute &quot;ALTER TABLE recipes VALIDATE CONSTRAINT cookbook_id_fkey&quot; , &quot;&quot; end These migrations can be in the same deployment, but make sure they are separate migrations. Adding a json column In Postgres, there is no equality operator for the json column type, which can cause errors for existing SELECT DISTINCT queries in your application. BAD ❌ def change do alter table ( &quot;recipes&quot; ) do add :extra_data , :json end end GOOD ✅ Use jsonb instead. Some say it’s like &quot;json&quot; but &quot; b etter.&quot; def change do alter table ( &quot;recipes&quot; ) do add :extra_data , :jsonb end end Keeping non-unique indexes to three columns or less BAD ❌ Adding a non-unique index with more than three columns rarely improves performance. defmodule Cookbook.AddIndexOnIngredients do def change do create index ( :recipes , [ :a , :b , :c , :d ] , concurrently : true ) end end GOOD ✅ Instead, start an index with columns that narrow down the results the most. defmodule Cookbook.AddIndexOnIngredients do def change do create index ( :recipes , [ :b , :d ] , concurrently : true ) end end For Postgres, be sure to add them concurrently.","ref":"readme.html#checks"},{"type":"extras","title":"Overview - Assuring safety","doc":"To mark an operation in a migration as safe use config comment. It will be ignored during analysis. There are two config comments available: excellent_migrations:safety-assured-for-next-line &lt;operation_type&gt; excellent_migrations:safety-assured-for-this-file &lt;operation_type&gt; Ignoring checks for given line: defmodule Cookbook.AddTypeToRecipesWithDefault do def change do alter table ( :recipes ) do # excellent_migrations:safety-assured-for-next-line column_added_with_default add ( :type , :string , default : &quot;dessert&quot; ) end end end Ignoring checks for the whole file: defmodule Cookbook.AddTypeToRecipesWithDefault do # excellent_migrations:safety-assured-for-this-file column_added_with_default def change do alter table ( :recipes ) do add ( :type , :string , default : &quot;dessert&quot; ) end end end Possible operation types are: check_constraint_added column_added_with_default column_reference_added column_removed column_renamed column_type_changed column_volatile_default index_concurrently_without_disable_ddl_transaction index_concurrently_without_disable_migration_lock index_not_concurrently json_column_added many_columns_index not_null_added operation_delete operation_insert operation_update raw_sql_executed table_dropped table_renamed","ref":"readme.html#assuring-safety"},{"type":"extras","title":"Overview - Disable checks","doc":"Ignore specific dangers for all migration checks with: config :excellent_migrations , skip_checks : [ :raw_sql_executed , :not_null_added ]","ref":"readme.html#disable-checks"},{"type":"extras","title":"Overview - Existing migrations","doc":"To skip analyzing migrations that were created before adding this package, set timestamp from the last migration in start_after in config: config :excellent_migrations , start_after : &quot;20191026080101&quot;","ref":"readme.html#existing-migrations"},{"type":"extras","title":"Overview - Similar tools &amp; resources","doc":"https://github.com/ankane/strong_migrations (Ruby) https://github.com/rrrene/credo (Elixir) https://github.com/fly-apps/safe-ecto-migrations – Special thanks for unsafe actions explanation and recipes. https://www.postgresql.org/docs/current/sql-altertable.html#Notes","ref":"readme.html#similar-tools-resources"},{"type":"extras","title":"Overview - Contributing","doc":"Everyone is encouraged and welcome to help improve this project. Here are a few ways you can help: Give feedback – your opinion matters Visit TODO list Submit pull request Suggest feature Report bug Improve documentation","ref":"readme.html#contributing"},{"type":"extras","title":"Overview - Copyright and License","doc":"Copyright (c) 2021 Artur Sulej This work is free. You can redistribute it and/or modify it under the terms of the MIT License. See the LICENSE.md file for more details.","ref":"readme.html#copyright-and-license"}]